#include "StdAfx.h"
#include <atlsafe.h>
#include "ContactHandler.h"

/*static*/ HRESULT CContactHandler::GetContact(const long nIndex, IDispatch* pContactOSP, ECtiClientLib::IECOMContact** ppContact)
{
	HRESULT hr = E_NOINTERFACE;
	ECtiClientLib::IECOMContactsPtr spContacts(pContactOSP);
	if(spContacts)
	{
		IUnknownPtr spUnk;
		hr = spContacts->get_Item(nIndex, &spUnk);
		if(spUnk)
			hr = spUnk->QueryInterface(__uuidof(*ppContact), (void**) ppContact);
	}

	return hr;
}

// IClientAddIn implementation
STDMETHODIMP CContactHandler::raw_GetAbout (/*out*/ BSTR* pbstrAbout, /*out*/ BSTR* pbstrInfo)
{
	*pbstrAbout = ::SysAllocString(L"SDKSample ContactHandler");
	*pbstrInfo = ::SysAllocString(L"Copyright 2010 ESTOS GmbH");
	return NOERROR;
}

// IContactHandler implementation
STDMETHODIMP CContactHandler::raw_OnRegisterHandler (
	/*[out, retval]*/ VARIANT* pvarAddInName)
{
	// return IContactHandlerImpl < CContactHandler >::OnRegisterHandler(pvarAddInName);

	CComSafeArray<BSTR> saStores;
	HRESULT hr = saStores.Create(3);
	hr = saStores.SetAt(0, L"conts");
	hr = saStores.SetAt(1, L"accts");
	hr = saStores.SetAt(2, L"leads");
	CComVariant vStores(saStores.Detach());
	hr = vStores.Detach(pvarAddInName);
	return hr;
};

// ACHTUNG:
//   die lebensdauer von pContactOSP wird durch die referenzzaehlung bestimmt.
//   wird hier eine asynchrone weiterverarbeitung angestossen die auf das NamedBinding des pContactOSP aufsetzt
//   so ist primaer der handler dafuer verantwortlich das, der pContactOSP zum zeitpunkt des GetObject() noch existiert.
//
// Loesungsansatz fuer eine "saubere" loesung:
//   eine zusaetzliche referenz zu erzeugen ist sehr einfach. Das problem ist "Wann" darf bzw. muss diese referenz freigegeben werden.
//   um nicht unntig speicher zu verbrauchen. am besten whre eine rckmeldung der weiterverarbeitenden instanz das die bindung
//   stattgefunden hat oder nicht mehr benoetigt wird. aufgrund dieser rueckmeldung kann die referenz "sicher" freigegeben werden.
//
// Beispiel:
//   wird der ContactHandler in einer Sprache realisiert die eine Automatische Speicherverwaltung untersttzt (Script, .NET, ...)
//   so ist in der Regel kein explizites eingreifen ntig.
//   Es ist eher die ausnahme das zwischen dem trigger der weiterverarbeitung und dem verlassen dieser methode ein
//   durchlauf des GarbageCollectors stattfindet.
//   ABER wer eine "saubere" loesung programmieren will MUSS die lebensdauer explizit steuern.
STDMETHODIMP CContactHandler::raw_OnContactNewExecute (
	/*[in]*/ BSTR       bstrFolderID,
	/*[in]*/ IDispatch* pContactOSP)
{
	// m_spContactOSP = pContactOSP;

	ECtiClientLib::IECOMContactPtr spContact;
	HRESULT hr = CContactHandler::GetContact(1, pContactOSP, &spContact);

	CComBSTR bstrURL = L"http://crm.estos.de:5555/ESTOS/sfa/";
	bstrURL += bstrFolderID; // conts | accts | leads
	bstrURL += L"/edit.aspx?new_urlparpe=";
	bstrURL += L"CLSID:432FED4F-B413-4040-A10F-ED92969A2AC0:!Contact:";
	CComBSTR bstrEntryID;
	hr = spContact->get_EntryID(&bstrEntryID);
	bstrURL += bstrEntryID + L";";
	CComBSTR bstrEntryIDStore;
	hr = spContact->get_EntryIDStore(&bstrEntryIDStore);
	bstrURL += bstrEntryIDStore + L";";
	CComBSTR bstrEntryIDDB;
	hr = spContact->get_EntryIDDB(&bstrEntryIDDB);
	bstrURL += bstrEntryIDDB + L":";

	// in our case
	// http://crm.estos.de:5555/ESTOS/sfa/conts/edit.aspx?new_urlparpe=CLSID:432FED4F-B413-4040-A10F-ED92969A2AC0:!Contact:Temporary;SDKSampleAddIn;DB24:
	// ::MessageBox(NULL, (LPCTSTR) bstrURL, _T("FromInsideAddIn::OnContactNewExecute()"), MB_OK | MB_ICONINFORMATION);
	::ShellExecute(NULL, _T("open"), (LPCTSTR) bstrURL, NULL, NULL, SW_SHOWNORMAL);
	return NOERROR;
}

STDMETHODIMP CContactHandler::raw_OnContactOpen(
	/*[in]*/ IDispatch* pContactOSP)
{
	// m_spContactOSP = pContactOSP;

	ECtiClientLib::IECOMContactPtr spContact;
	HRESULT hr = CContactHandler::GetContact(1, pContactOSP, &spContact);

	CComBSTR bstrURL = L"http://crm.estos.de:5555/ESTOS/sfa/conts/edit.aspx?id=";
	CComBSTR bstrEntryID;
	hr = spContact->get_EntryID(&bstrEntryID);
	bstrURL += bstrEntryID;

	bstrURL += L"&new_urlparpe=CLSID:432FED4F-B413-4040-A10F-ED92969A2AC0:!Contact:";
	bstrURL += bstrEntryID + L";";
	CComBSTR bstrEntryIDStore;
	hr = spContact->get_EntryIDStore(&bstrEntryIDStore);
	bstrURL += bstrEntryIDStore + L";";
	CComBSTR bstrEntryIDDB;
	hr = spContact->get_EntryIDDB(&bstrEntryIDDB);
	bstrURL += bstrEntryIDDB + L":";

	// ::MessageBox(NULL, (LPCTSTR) bstrURL, _T("FromInsideAddIn::OnContactOpen()"), MB_OK | MB_ICONINFORMATION);
	::ShellExecute(NULL, _T("open"), (LPCTSTR) bstrURL, NULL, NULL, SW_SHOWNORMAL);
	return NOERROR;
}

STDMETHODIMP CContactHandler::raw_OnGetContactByID(
	/*[in]*/ IDispatch* pContactOSP)
{
	// hier muss jetzt eine abfrage an den MSCRM Server ueber SOAP bzw. das MSCRM SDK abgeschikt werden
	// und die felder in den ECOMContact gemapped werden
	ECtiClientLib::IECOMContactPtr spContact;
	HRESULT hr = CContactHandler::GetContact(1, pContactOSP, &spContact);

	// http://msdn.microsoft.com/en-us/dynamics/crm/default.aspx
	// Add Web Reference
	//   http://crm.estos.de:5555/mscrmservices/2007/CrmServiceWsdl.aspx?uniquename=ESTOS
	return hr;
}
